@@ -75,10 +75,22 @@ fitted.HLfitlist <- function(object,...) {
   resu
 }
 
-logLik.HLfitlist <- function(object, REML = FALSE, ...) {
-  if (REML) {
-    return(object$APHLs$p_bv)
-  } else {
-    return(object$APHLs$p_v)
-  }    
+logLik.HLfitlist <- function(object, which = NULL, ...) {
+  item <- getHLfit(object[[1]])
+  if (is.null(which)) {
+    mess <- REMLmess(item)
+    which <- switch(mess, 
+                    "by stochastic EM."= "logLapp",
+                    "by ML approximation (p_v)."= "p_v",
+                    "by h-likelihood approximation."= "p_v",
+                    "by ML."= "p_v",
+                    "by REML approximation (p_bv)."= "p_bv",
+                    "by REML."= "p_bv",
+                    "by non-standard REML"= "p_bv",
+                    stop(paste("No default '",which,"' value for '",mess,"' estimation method.",sep=""))
+    ) 
+  }
+  resu  <- attr(object,"APHLs")[[which]]
+  names(resu) <- which
+  return(resu)
 }  
